close all

%% read data
[ Settings, Data ] = Read_RFe_V2c( 'P:\TNW\BMPI\Users\Ho Nhu Y Nguyen\Published papers\In-plane artifact reduction\data', 0, 0);
nRows = Settings.halfFrameLength;
nCols = Settings.finfo.nrl;
nPixels = nCols*nRows;

%% preprocess data
Settings.events = unique(Settings.lm(4,:));
if Settings.events(1) == 0
    Settings.events(1) = [];
end
nEvents = length(Settings.events);
signal = preprocess_data_PA(Settings, Data);

%% reconstruct images
fprintf('Reconstructing image...');
Settings.pitch = 0.3; % transducer element Settings.pitch [mm]
Settings.c =  1.5; % speed of sound mm/s
Settings.delay = 1; % time delay from laser irradiation to signal acquisition start
rekon = zeros(nEvents, Settings.halfFrameLength, Settings.finfo.nrl);
Settings.sampleDepth = Settings.c*1e6/Settings.acq.fs; % depth of each sample
sampleDepth = Settings.sampleDepth;
Settings.acqDepth = [1, Settings.halfFrameLength]*Settings.sampleDepth;

maxRekon = 0;
clearestImgIdx = 1; % to find the clearest image (highest SRN)
figure
subplotRow = ceil(sqrt(nEvents));
subplotCol = ceil(nEvents/subplotRow);
for iEvent = 1:nEvents
    subplot(subplotRow, subplotCol, iEvent);
    tempRekon = rekon_OA_freqdom(squeeze(signal(iEvent,:,:)), Settings);
    rekon(iEvent,:,:) = tempRekon;
    if max(tempRekon(:)) > maxRekon
        maxRekon = max(tempRekon(:));
        clearestImgIdx = iEvent;
    end
    imagesc(tempRekon);
    ylabel('depth [mm]');
    xlabel('line index');
    colormap(hot);
    colorbar;
    rekon(iEvent,:,:) = tempRekon;
    title(num2str(Settings.events(iEvent)));
end
fprintf('done!\n');
clearestImg = squeeze(rekon(clearestImgIdx, :, :));

% profile on
corrThreshold = 0.95;
depthThreshold = floor(2/sampleDepth);
startScanIdx = 125*nCols; % first 50 lines are skipped
stopScanIdx = nCols*(nRows-depthThreshold);

% to optimize the algorithm, the image is scanned row by row instead of
% column by column as Matlab does by default
clearestImg = clearestImg';
imgIds = 1:nPixels;
imgIds = reshape(imgIds, nRows, nCols);
imgIds = imgIds';
imgIds = reshape(imgIds, [], 1);

allPxlResp = reshape(rekon, nEvents, nPixels);
allPxlResp = allPxlResp(:, imgIds);
meanAllPxlResp = sum(allPxlResp)/nEvents;
allMinusMeanAllPxlResp = allPxlResp-meanAllPxlResp;
stdAllPxlResp = std(allPxlResp);
allPxlCoef = allMinusMeanAllPxlResp./stdAllPxlResp;
reflMask = false(1, nPixels);
clearestImg = reshape(clearestImg, 1, nPixels);
allPxlY = repmat((1:nCols)*Settings.pitch,1,nRows);
allPxlY = reshape(allPxlY, 1, nPixels);
allPxlZ = repmat((1:nRows)*sampleDepth,nCols,1);
allPxlZ = reshape(allPxlZ, 1, nPixels);
curMask = false(1, nPixels);

DevRemoveObj = struct([]);

%% pixel by pixel
for iScan = startScanIdx:stopScanIdx % scan row by row
% iScan = (iRow-1)*64+iCol
% For example, if you want to debug/analyze pixel at (163,30), put a bug and
% set a condition as iScan=(163-1)*64+30=10398. When the script reaches
% that bug, you can see the corelation coefficient map of whole image with
% that pixel.
    iPxl = imgIds(iScan);
    if reflMask(iScan)
        continue
    end
    iCol = ceil(iPxl/nRows);
    iRow = iPxl-(iCol-1)*nRows;
    iDepthThreshold = iRow+depthThreshold;    
    
    curCoef = allPxlCoef.*allPxlCoef(:,iScan);
    corrCoef = sum(curCoef)/(nEvents-1);
    corrMask = corrCoef > corrThreshold;
    
    curZ = iRow*sampleDepth;
    curY = iCol*Settings.pitch;
    curMask(:) = true;
    
    % check the z distance
    curMask = reshape(curMask, nCols, nRows);
    curMask(:,1:iDepthThreshold) = false;
    curMask = reshape(curMask,1,nPixels);
    
    % combine all conditions
    curMask = curMask & (clearestImg<clearestImg(iScan)); % the signal from
                      % the reflections must be weaker than the real signal
    curMask = curMask & corrMask;
    curMask = curMask & ~reflMask; % not take the identified reflections again
    reflMask(curMask) = true;
    %% dev part
    if ~isempty(find(curMask>0,1))
        reflections = curMask;
        reflections = reshape(reflections, nCols, nRows);
        reflections = reflections';
        DevRemoveObj(end+1).absLin = iPxl;
        DevRemoveObj(end).absX = iCol;
        DevRemoveObj(end).absY = iRow;
        DevRemoveObj(end).reflLin = find(reflections>0);
        DevRemoveObj(end).reflX = ceil(DevRemoveObj(end).reflLin/nRows);
        DevRemoveObj(end).reflY = DevRemoveObj(end).reflLin -(DevRemoveObj(end).reflX-1)*nRows;
        DevRemoveObj(end).absPxlValue = clearestImg(iScan);
        % find the corresponding absorber        
%         find([DevRemoveObj.absX] == 35 & [DevRemoveObj.absY] == 159)
%         a = arrayfun(@(x)find(x.reflX == 33 & x.reflY == 234),DevRemoveObj, 'uniformoutput',false);
%         a(~cellfun('isempty',a));
    end
end
%% correct the image
clearestImg = reshape(clearestImg, nCols, nRows);
correctedImg = clearestImg;
correctedImg(reflMask) = 0;
clearestImg = clearestImg';
correctedImg = correctedImg';

% profile viewer
indices = Settings.lm(4,:) == 0;
frames = Data.fullFrame(indices,:,:);
tempFrame = squeeze(mean(frames));
tempFrame(Settings.cutOffIdx1+1:end-Settings.cutOffFullIdx-1, :) = detrend(tempFrame(Settings.cutOffIdx1+1:end-Settings.cutOffFullIdx-1, :), 'constant');
USFrame = rekon_US_freqdom_anglenew_compounding(tempFrame...
        ,Settings.acq.fs/1e6,0.3,Settings.c,2,0,0,5,1,1e9,0);
% USFrame(274:721,:) = USFrame(253:700,:);

figure
subplot(1,3,1)
imagesc(1:64*Settings.pitch, Settings.acqDepth, USFrame);
ylabel('depth [mm]');
xlabel('y [mm]');
title('US image');
colormap hot
colorbar
axis image
subplot(1,3,2)
% imagesc(clearestImg);
imagesc(1:64*Settings.pitch, Settings.acqDepth, clearestImg);
axis image
ylabel('depth [mm]');
xlabel('y [mm]');
title('Original image');
colormap hot
colorbar
subplot(1,3,3)
% imagesc(correctedImg)
imagesc(1:64*Settings.pitch, Settings.acqDepth, correctedImg);
axis image
ylabel('depth [mm]');
xlabel('y [mm]');
title('Corrected image');
colormap hot
colorbar
fprintf('done!\n');

