close all

%% read data
[ Settings, Data ] = Read_RFe_V2c( 'D:\CVENT\test data\multi-wavelength probe\Y and Onno\2018_06_11 occlusion', 0, 0);

%% preprocess data
% profile on
Settings.events = unique(Settings.lm(4,:));
if Settings.events(1) == 0
    Settings.events(1) = [];
end
wavelengths = wavelength_event_converter(Settings.events);
nEvents = length(Settings.events);
signal = preprocess_data_PA(Settings, Data);
% figure
% imagesc(squeeze(signal))
% colormap hot
%% reconstruct images
fprintf('Reconstructing image...');
Settings.pitch = 0.3; % transducer element Settings.pitch [mm]
Settings.c =  1.5; % speed of sound mm/s
Settings.delay = 1; % time delay from laser irradiation to signal acquisition start
rekon = zeros(nEvents, Settings.halfFrameLength, Settings.finfo.nrl);
Settings.sampleDepth = Settings.c*1e6/Settings.acq.fs; % depth of each sample
Settings.acqDepth = [1, Settings.halfFrameLength]*Settings.sampleDepth;

maxRekon = 0;
clearestImgIdx = 1; % to find the clearest image (highest SRN)
figure
subplotRow = ceil(sqrt(nEvents));
subplotCol = ceil(nEvents/subplotRow);
for iEvent = 1:nEvents
    subplot(subplotRow, subplotCol, iEvent);
%     subplot(4, 1, iEvent);
    tempRekon = rekon_OA_freqdom(squeeze(signal(iEvent,:,:)), Settings);
%     plot(tempRekon(:, 36));
%     plot(squeeze(signal(iEvent,:,36)), '--');
    rekon(iEvent,:,:) = tempRekon;
    if max(tempRekon(:)) > maxRekon
        maxRekon = max(tempRekon(:));
        clearestImgIdx = iEvent;
    end
%     imagesc(1:64*Settings.pitch, Settings.acqDepth, tempRekon);
    axis image
    imagesc(tempRekon);
    ylabel('depth [mm]');
    xlabel('line index');
    colormap(hot);
    colorbar;
%     set(gca, 'CLim', [0, 130]);
    rekon(iEvent,:,:) = tempRekon;
    title(['\lambda = ' wavelengths{iEvent} 'nm']);
end
fprintf('done!\n');
clearestImg = squeeze(rekon(clearestImgIdx, :, :));
indices = Settings.lm(4,:) == 0;
frames = Data.fullFrame(indices,:,:);
tempFrame = squeeze(mean(frames));
tempFrame(Settings.cutOffIdx1+1:end-Settings.cutOffFullIdx-1, :) = detrend(tempFrame(Settings.cutOffIdx1+1:end-Settings.cutOffFullIdx-1, :), 'constant');
USFrame = rekon_US_freqdom_anglenew_compounding(tempFrame...
        ,Settings.acq.fs/1e6,0.3,Settings.c,2,0,0,5,1,1e9,0);
USFrame(274:721,:) = USFrame(253:700,:);
figure
subplot(1,2,1)
imagesc(USFrame);
% imagesc((1:64)*Settings.pitch, Settings.acqDepth, USFrame);
% axis image
ylabel('depth [mm]');
xlabel('x [a.u]');
title('US image');
colormap hot
colorbar

subplot(1,2,2)
% imagesc(clearestImg)
imagesc((1:64)*Settings.pitch, Settings.acqDepth, clearestImg);
axis image
ylabel('depth [mm]');
xlabel('x [a.u]');
title('Original image');
colormap hot
colorbar